# Default options: --with sound --with dlopen --with xscreensaver
#
# --without sound:  disables all sound support.
# --without dlopen: links with OpenAL and libvorbisfile at compile-time,
#                   rather than deferring it to run-time.
# --without xscreensaver:
#                   produce stand-alone programs only.

%{?_with_sound: %{?_without_sound:%{error:both '--with sound' and '--without sound' specified}}}
%{?_with_dlopen:%{?_without_sound:%{error:both '--with dlopen' and '--without dlopen' specified}}}
%{?_with_xscreensaver: %{?_without_xscreensaver:%{error:both '--with xscreensaver' and '--without xscreensaver' specified}}}

%{?_without_sound: %define _configure_sound --disable-sound}
%{!?_without_sound:%define _configure_sound --enable-sound%{!?_without_dlopen:=dlopen}}

# Expand hackdir -- configure leaves this unexpanded so that
# one could do "make install exec_prefix=..."
%define _hackdir %(exec_prefix="%{_exec_prefix}"; echo "${exec_prefix}/bin")

# Sanity check: don't allow --with xscreensaver if xscreensaver wasn't
# around when rsxs.spec was generated by configure, because we really don't
# know where files are supposed to go...
%if "no" == "yes" && 0%{!?_without_xscreensaver:1}

	%define _defaultdir 
	%define _configdir  
	%define __xscreensaver_command /usr/bin/xscreensaver-command

	%define _configure_xscreensaver --with-xscreensaver --with-defaultdir=%{_defaultdir} --with-hackdir=%{_hackdir} --with-configdir=%{_configdir}
	%define _use_xscreensaver 1

%else

	%define _configure_xscreensaver --without-xscreensaver

%endif

%define _configure_opts %{_configure_sound} %{_configure_xscreensaver} --enable-image

# %{dist} and %{fedora} are reliable, and they're used in mock.
# But if we don't have %{fedora}, try to determine it programmatically...
%if "0%{?fedora}" == "0"
%define fedora %(awk '{print $4}' /etc/fedora-release 2>/dev/null || echo 0)
%endif

Name: rsxs
Version: 0.9
Release: 1%{?dist}
License: GPL
Source0: rsxs-0.9.tar.gz
URL: http://rsxs.sourceforge.net/
BuildRoot: %{_tmppath}/%{name}-%{version}-root
%if "%{fedora}" >= "5"
BuildRequires: libSM-devel, libICE-devel, libXmu-devel, libXt-devel
%else
BuildRequires: XFree86-devel
%endif
BuildRequires: libGL-devel, libGLU-devel, libpng-devel
%{!?_without_sound:BuildRequires: openal-devel, libvorbis-devel}
Summary: Really Slick XScreenSavers
Group: Amusements/Graphics
Requires: mktemp%{?_use_xscreensaver:, xscreensaver}
Obsoletes: rsxs-nosound
Provides: rsxs-nosound

%description
The Really Slick XScreenSavers package is an X11/GLX port of the
Really Slick Screensavers collection, by Terry Welsh
(http://www.reallyslick.com).

%prep
%setup -q -n rsxs-0.9

%build
%{configure} %{_configure_opts}
%{__make}

%install
if test "$RPM_BUILD_ROOT" != "/"; then
	%{__rm} -rf $RPM_BUILD_ROOT
fi
%{makeinstall} \
	%{?_defaultdir:defaultdir=$RPM_BUILD_ROOT%{_defaultdir}} \
	hackdir=$RPM_BUILD_ROOT%{_hackdir} \
	%{?_configdir:configdir=$RPM_BUILD_ROOT%{_configdir}}

%clean
if test "$RPM_BUILD_ROOT" != "/"; then
	%{__rm} -rf $RPM_BUILD_ROOT
fi

%if 0%{?_use_xscreensaver}

%post
echo "Installing screensavers into %{_defaultdir}/XScreenSaver"
TMPFILE1=`mktemp %{_tmppath}/$$.XXXXXX` || exit 1;
TMPFILE2=`mktemp %{_tmppath}/$$.XXXXXX` || exit 1;
trap "%{__rm} -f $TMPFILE1 $TMPFILE2" 0
%{__cat} <<"EOF" > $TMPFILE1
BEGIN {
	if (skip !~ /cyclone/)
		add["rs-cyclone"] =                           "           GL:  \"Really Slick Cyclones\" rs-cyclone -root                    \\n\\"
	
	if (skip !~ /euphoria/) {
		add["\\(regular\\)\" rs-euphoria"] =          "           GL:  \"Really Slick Euphoria (regular)\" rs-euphoria -stringy -root \\n\\"
		add["\\(grid\\)\" rs-euphoria"] =             "           GL:  \"Really Slick Euphoria (grid)\" rs-euphoria             -wisps 4  -background 1 -density 25 -visibility 70 -speed 15 -wireframe -root \\n\\"
		add["\\(cubism\\)\" rs-euphoria"] =           "           GL:  \"Really Slick Euphoria (cubism)\" rs-euphoria           -wisps 15 -background 0 -density 4  -visibility 15 -speed 10 -root \\n\\"
		add["\\(bad math\\)\" rs-euphoria"] =         "           GL:  \"Really Slick Euphoria (bad math)\" rs-euphoria         -wisps 2  -background 2 -density 20 -visibility 35 -speed 30 -feedback 40 -feedbacksize 5 -feedbacksize 8 -wireframe -lines -root \\n\\"
		add["\\(M-theory\\)\" rs-euphoria"] =         "           GL:  \"Really Slick Euphoria (M-theory)\" rs-euphoria         -wisps 3  -background 0 -density 25 -visibility 35 -speed 20 -feedback 40 -feedbackspeed 20 -feedbacksize 8 -root \\n\\"
		add["\\(UHF TEM\\)\" rs-euphoria"] =          "           GL:  \"Really Slick Euphoria (UHF TEM)\" rs-euphoria          -wisps 0  -background 3 -density 35 -visibility 5  -speed 50 -root \\n\\"
		add["\\(nowhere\\)\" rs-euphoria"] =          "           GL:  \"Really Slick Euphoria (nowhere)\" rs-euphoria          -wisps 0  -background 3 -density 30 -visibility 40 -speed 20 -feedback 80 -feedbackspeed 10 -feedbacksize 8 -wireframe -random -root \\n\\"
		add["\\(echo\\)\" rs-euphoria"] =             "           GL:  \"Really Slick Euphoria (echo)\" rs-euphoria             -wisps 3  -background 0 -density 25 -visibility 30 -speed 20 -feedback 85 -feedbackspeed 30 -feedbacksize 8 -plasma -root \\n\\"
		add["\\(kaleidoscope\\)\" rs-euphoria"] =     "           GL:  \"Really Slick Euphoria (kaleidoscope)\" rs-euphoria     -wisps 3  -background 0 -density 25 -visibility 40 -speed 15 -feedback 90 -feedbackspeed 3  -feedbacksize 8 -root \\n\\"
	}
	
	if (skip !~ /fieldlines/)
		add["rs-fieldlines"] =                        "           GL:  \"Really Slick Fieldlines\" rs-fieldlines -root               \\n\\"
	
	if (skip !~ /flocks/)
		add["rs-flocks"] =                            "           GL:  \"Really Slick Flocks\" rs-flocks -root                       \\n\\"
	
	if (skip !~ /flux/) {
		add["\\(regular\\)\" rs-flux"] =              "           GL:  \"Really Slick Flux (regular)\" rs-flux -root                 \\n\\"
		add["\\(hypnotic\\)\" rs-flux"] =             "           GL:  \"Really Slick Flux (hypnotic)\" rs-flux                 -fluxes 2  -particles 10 -length 40 -lights  -size 15  -randomness 80 -speed 20 -rotation 0  -wind 40 -instability 10  -blur 30 -root \\n\\"
		add["\\(insane\\)\" rs-flux"] =               "           GL:  \"Really Slick Flux (insane)\" rs-flux                   -fluxes 4  -particles 30 -length 8  -lights  -size 25  -randomness 0  -speed 80 -rotation 60 -wind 40 -instability 100 -blur 10 -root \\n\\"
		add["\\(sparklers\\)\" rs-flux"] =            "           GL:  \"Really Slick Flux (sparklers)\" rs-flux                -fluxes 3  -particles 20 -length 6  -spheres -size 20  -randomness 85 -speed 60 -rotation 30 -wind 20 -instability 30  -blur 0  -root \\n\\"
		add["\\(paradigm\\)\" rs-flux"] =             "           GL:  \"Really Slick Flux (paradigm)\" rs-flux                 -fluxes 1  -particles 40 -length 40 -lights  -size 5   -randomness 90 -speed 30 -rotation 20 -wind 10 -instability 5   -blur 10 -root \\n\\"
		add["\\(fusion\\)\" rs-flux"] =               "           GL:  \"Really Slick Flux (fusion)\" rs-flux                   -fluxes 10 -particles 3  -length 10 -lights  -size 100 -randomness 0  -speed 50 -rotation 30 -wind 40 -instability 35  -blur 50 -root \\n\\"
	}
	
	if (skip !~ /helios/)
		add["rs-helios"] =                            "           GL:  \"Really Slick Helios\" rs-helios -root                       \\n\\"
	
	if (skip !~ /hyperspace/)
		add["rs-hyperspace"] =                        "           GL:  \"Really Slick Hyperspace\" rs-hyperspace -root               \\n\\"
	
	if (skip !~ /lattice/) {
		add["\\(regular\\)\" rs-lattice"] =           "           GL:  \"Really Slick Lattice (regular)\" rs-lattice -root           \\n\\"
		add["\\(chainmail\\)\" rs-lattice"] =         "           GL:  \"Really Slick Lattice (chainmail)\" rs-lattice          -longitude 24 -latitude 12 -thickness 50  -density 80 -depth 3 -chrome   -smooth -root \\n\\"
		add["\\(brass mesh\\)\" rs-lattice"] =        "           GL:  \"Really Slick Lattice (brass mesh)\" rs-lattice         -longitude 4  -latitude 4  -thickness 40  -density 50 -depth 4 -brass    -no-smooth -root \\n\\"
		add["\\(computer\\)\" rs-lattice"] =          "           GL:  \"Really Slick Lattice (computer)\" rs-lattice           -longitude 4  -latitude 6  -thickness 70  -density 90 -depth 4 -circuits -no-smooth -root \\n\\"
		add["\\(slick\\)\" rs-lattice"] =             "           GL:  \"Really Slick Lattice (slick)\" rs-lattice              -longitude 24 -latitude 12 -thickness 100 -density 30 -depth 4 -shiny    -smooth -root \\n\\"
		add["\\(tasty\\)\" rs-lattice"] =             "           GL:  \"Really Slick Lattice (tasty)\" rs-lattice              -longitude 24 -latitude 12 -thickness 100 -density 25 -depth 4 -donuts   -smooth -root \\n\\"
	}
	
	if (skip !~ /plasma/)
		add["rs-plasma"] =                            "           GL:  \"Really Slick Plasma\" rs-plasma -root                       \\n\\"
	
	if (skip !~ /skyrocket/)
		add["rs-skyrocket"] =                         "           GL:  \"Really Slick Skyrocket\" rs-skyrocket -volume 0 -root                 \\n\\"
	
	if (skip !~ /solarwinds/) {
		add["\\(regular\\)\" rs-solarwinds"] =        "           GL:  \"Really Slick Solar Winds (regular)\" rs-solarwinds -root    \\n\\"
		add["\\(cosmic strings\\)\" rs-solarwinds"] = "           GL:  \"Really Slick Solar Winds (cosmic strings)\" rs-solarwinds -winds 1 -emitters 50  -particles 3000 -lines  -size 20 -windspeed 10  -emitterspeed 10  -speed 10  -blur 10 -root \\n\\"
		add["\\(cold pricklies\\)\" rs-solarwinds"] = "           GL:  \"Really Slick Solar Winds (cold pricklies)\" rs-solarwinds -winds 1 -emitters 300 -particles 3000 -lines  -size 5  -windspeed 20  -emitterspeed 100 -speed 15  -blur 70 -root \\n\\"
		add["\\(space fur\\)\" rs-solarwinds"] =      "           GL:  \"Really Slick Solar Winds (space fur)\" rs-solarwinds      -winds 2 -emitters 400 -particles 1600 -lines  -size 15 -windspeed 20  -emitterspeed 15  -speed 10  -blur 0  -root \\n\\"
		add["\\(jiggly\\)\" rs-solarwinds"] =         "           GL:  \"Really Slick Solar Winds (jiggly)\" rs-solarwinds         -winds 1 -emitters 40  -particles 1200 -points -size 20 -windspeed 100 -emitterspeed 20  -speed 4   -blur 50 -root \\n\\"
		add["\\(undertow\\)\" rs-solarwinds"] =       "           GL:  \"Really Slick Solar Winds (undertow)\" rs-solarwinds       -winds 1 -emitters 400 -particles 1200 -lights -size 40 -windspeed 20  -emitterspeed 1   -speed 100 -blur 50 -root \\n\\"
	}
}

/^[ \t]*\*programs:/ {
	print
	output = ""
	do {
		getline
		output = output $0 ORS
		x = 0
		for (name in add)
			if ($0 ~ name)
				remove[++x] = name
		for (; x > 0; --x)
			delete add[remove[x]]
	} while ($0 ~ /\\$/)
	for (name in add) {
		print add[name]
	}
	ORS=""
	print output
	next
}

! /^[ \t]*\*programs:/ {
	ORS="\n"
	print
}
EOF
%{__awk} -f $TMPFILE1 < %{_defaultdir}/XScreenSaver > $TMPFILE2
%{__chmod} a+r $TMPFILE2
%{__mv} -f $TMPFILE2 %{_defaultdir}/XScreenSaver
# Can't just use return code from xscreensaver-command due to
# "xscreensaver window unexpectedly deleted" quirk
if %{__xscreensaver_command} -version &> /dev/null; then
	echo "Restarting XScreenSaver"
	%{__xscreensaver_command} --restart &> /dev/null || :
fi

%preun
if test $1 -eq 0; then
	echo "Removing screensavers from %{_defaultdir}/XScreenSaver"
	TMPFILE1=`mktemp %{_tmppath}/$$.XXXXXX` || exit 1;
	TMPFILE2=`mktemp %{_tmppath}/$$.XXXXXX` || exit 1;
	trap "%{__rm} -f $TMPFILE1 $TMPFILE2" 0
	%{__cat} <<"EOF" > $TMPFILE1
BEGIN {
	if (skip !~ /cyclone/)
		remove["rs-cyclone"] = 1
	
	if (skip !~ /euphoria/) {
		remove["\\(regular\\)\" rs-euphoria"] = 1
		remove["\\(grid\\)\" rs-euphoria"] = 1
		remove["\\(cubism\\)\" rs-euphoria"] = 1
		remove["\\(bad math\\)\" rs-euphoria"] = 1
		remove["\\(M-theory\\)\" rs-euphoria"] = 1
		remove["\\(UHF TEM\\)\" rs-euphoria"] = 1
		remove["\\(nowhere\\)\" rs-euphoria"] = 1
		remove["\\(echo\\)\" rs-euphoria"] = 1
		remove["\\(kaleidoscope\\)\" rs-euphoria"] = 1
	}
	
	if (skip !~ /fieldlines/)
		remove["rs-fieldlines"] = 1
	
	if (skip !~ /flocks/)
		remove["rs-flocks"] = 1
	
	if (skip !~ /flux/) {
		remove["\\(regular\\)\" rs-flux"] = 1
		remove["\\(hypnotic\\)\" rs-flux"] = 1
		remove["\\(insane\\)\" rs-flux"] = 1
		remove["\\(sparklers\\)\" rs-flux"] = 1
		remove["\\(paradigm\\)\" rs-flux"] = 1
		remove["\\(fusion\\)\" rs-flux"] = 1
	}
	
	if (skip !~ /helios/)
		remove["rs-helios"] = 1
	
	if (skip !~ /hyperspace/)
		remove["rs-hyperspace"] = 1
	
	if (skip !~ /lattice/) {
		remove["\\(regular\\)\" rs-lattice"] = 1
		remove["\\(chainmail\\)\" rs-lattice"] = 1
		remove["\\(brass mesh\\)\" rs-lattice"] = 1
		remove["\\(computer\\)\" rs-lattice"] = 1
		remove["\\(slick\\)\" rs-lattice"] = 1
		remove["\\(tasty\\)\" rs-lattice"] = 1
	}
	
	if (skip !~ /plasma/)
		remove["rs-plasma"] = 1
	
	if (skip !~ /skyrocket/)
		remove["rs-skyrocket"] = 1
	
	if (skip !~ /solarwinds/) {
		remove["\\(regular\\)\" rs-solarwinds"] = 1
		remove["\\(cosmic strings\\)\" rs-solarwinds"] = 1
		remove["\\(cold pricklies\\)\" rs-solarwinds"] = 1
		remove["\\(space fur\\)\" rs-solarwinds"] = 1
		remove["\\(jiggly\\)\" rs-solarwinds"] = 1
		remove["\\(undertow\\)\" rs-solarwinds"] = 1
	}
}

/^[ \t]*\*programs:/ {
	print
	output = ""
	do {
		getline
		doRemove = ""
		for (name in remove)
			if ($0 ~ name)
				doRemove = name
		if (doRemove)
			delete remove[doRemove]
		else
			output = output $0 ORS
	} while ($0 ~ /\\$/)
	ORS=""
	print output
	next
}

! /^[ \t]*\*programs:/ {
	ORS="\n"
	print
}
EOF
	%{__awk} -f $TMPFILE1 < %{_defaultdir}/XScreenSaver > $TMPFILE2
	%{__chmod} a+r $TMPFILE2
	%{__mv} -f $TMPFILE2 %{_defaultdir}/XScreenSaver
	# Can't just use return code from xscreensaver-command due to
	# "xscreensaver window unexpectedly deleted" quirk
	if %{__xscreensaver_command} -version &> /dev/null; then
		echo "Restarting XScreenSaver"
		%{__xscreensaver_command} --restart &> /dev/null || :
	fi
fi

%verifyscript
echo "Checking screensavers are in %{_defaultdir}/XScreenSaver"
TMPFILE1=`mktemp %{_tmppath}/$$.XXXXXX` || exit 1;
TMPFILE2=`mktemp %{_tmppath}/$$.XXXXXX` || exit 1;
trap "%{__rm} -f $TMPFILE1 $TMPFILE2" 0
%{__cat} <<"EOF" > $TMPFILE1
BEGIN {
	if (skip !~ /cyclone/)
		verify["rs-cyclone"] = 1
	
	if (skip !~ /euphoria/) {
		verify["\\(regular\\)\" rs-euphoria"] = 1
		verify["\\(grid\\)\" rs-euphoria"] = 1
		verify["\\(cubism\\)\" rs-euphoria"] = 1
		verify["\\(bad math\\)\" rs-euphoria"] = 1
		verify["\\(M-theory\\)\" rs-euphoria"] = 1
		verify["\\(UHF TEM\\)\" rs-euphoria"] = 1
		verify["\\(nowhere\\)\" rs-euphoria"] = 1
		verify["\\(echo\\)\" rs-euphoria"] = 1
		verify["\\(kaleidoscope\\)\" rs-euphoria"] = 1
	}
	
	if (skip !~ /fieldlines/)
		verify["rs-fieldlines"] = 1
	
	if (skip !~ /flocks/)
		verify["rs-flocks"] = 1
	
	if (skip !~ /flux/) {
		verify["\\(regular\\)\" rs-flux"] = 1
		verify["\\(hypnotic\\)\" rs-flux"] = 1
		verify["\\(insane\\)\" rs-flux"] = 1
		verify["\\(sparklers\\)\" rs-flux"] = 1
		verify["\\(paradigm\\)\" rs-flux"] = 1
		verify["\\(fusion\\)\" rs-flux"] = 1
	}
	
	if (skip !~ /helios/)
		verify["rs-helios"] = 1
	
	if (skip !~ /hyperspace/)
		verify["rs-helios"] = 1
	
	if (skip !~ /lattice/) {
		verify["\\(regular\\)\" rs-lattice"] = 1
		verify["\\(chainmail\\)\" rs-lattice"] = 1
		verify["\\(brass mesh\\)\" rs-lattice"] = 1
		verify["\\(computer\\)\" rs-lattice"] = 1
		verify["\\(slick\\)\" rs-lattice"] = 1
		verify["\\(tasty\\)\" rs-lattice"] = 1
	}
	
	if (skip !~ /plasma/)
		verify["rs-plasma"] = 1
	
	if (skip !~ /skyrocket/)
		verify["rs-skyrocket"] = 1
	
	if (skip !~ /solarwinds/) {
		verify["\\(regular\\)\" rs-solarwinds"] = 1
		verify["\\(cosmic strings\\)\" rs-solarwinds"] = 1
		verify["\\(cold pricklies\\)\" rs-solarwinds"] = 1
		verify["\\(space fur\\)\" rs-solarwinds"] = 1
		verify["\\(jiggly\\)\" rs-solarwinds"] = 1
		verify["\\(undertow\\)\" rs-solarwinds"] = 1
	}
}

/^[ \t]*\*programs:/ {
	output = ""
	do {
		getline
		doRemove = ""
		for (name in verify)
			if ($0 ~ name)
				doVerify = name
		if (doVerify)
			delete verify[doVerify]
		output = output $0 ORS
	} while ($0 ~ /\\$/)
	next
}

END {
	ORS="\n"
	count = 0
	for (name in verify) {
		++count
		print name
	}
	exit count
}
EOF
%{__awk} -f $TMPFILE1 < %{_defaultdir}/XScreenSaver > $TMPFILE2
if test $? -eq 0; then
	echo "All screensavers correctly installed"
else
	echo "Some screensavers not correctly installed"
	echo "%{_defaultdir}/XScreenSaver does not have the following screensavers:"
	%{__cat} $TMPFILE2
fi

%endif

%files
%defattr(-,root,root)
%doc AUTHORS COPYING ChangeLog INSTALL README THANKS
%if 0%{?_use_xscreensaver}
%{_hackdir}/rs-*
%{_configdir}/rs-*
%endif
%{_bindir}/rs-*
%{_datadir}/rsxs

%changelog
* Sat Jul 15 2005 Michael Chapman <foonly@users.sourceforge.net> 0.9-1
- fixed skyrocket memory allocation
- fixed xscreensaver config files
- added hyperspace screensaver
- package overhaul; it now actually builds on modern OSs!
- added libltdl so sound can be dynamically loaded; this simplifies
  the RPM packaging considerably

* Fri Jul 11 2003 Michael Chapman <foonly@users.sourceforge.net> 0.8-1
- major package restructure
- added skyrocket screensaver

* Wed Dec 11 2002 Michael Chapman <foonly@users.sourceforge.net> 0.7-1
- added flocks screensaver
- added flux screensaver
- added solarwinds screensaver
- cleaned up command-line argument handling
- added support for pre-4.0 versions of xscreensaver

* Sat Dec 07 2002 Michael Chapman <foonly@users.sourceforge.net> 0.6-1
- added lattice screensaver
- converted textures to PNG format
- small Makefile fix to build correctly on Gentoo

* Tue Nov 28 2002 Michael Chapman <foonly@users.sourceforge.net> 0.5-1
- added euphoria screensaver
- fixed bug in timing delay
- removed unnecessary #include <ios>

* Tue Nov 27 2002 Michael Chapman <foonly@users.sourceforge.net> 0.4-2
- fixed uninstall script -- should upgrade properly now
- fixed bug in timing delay

* Tue Nov 26 2002 Michael Chapman <mchapman@student.usyd.edu.au> 0.4-1
- added helios screensaver
- expanded screensaver descriptions
- removed unnecessary labels

* Mon Nov 19 2002 Michael Chapman <mchapman@student.usyd.edu.au> 0.3-1
- added fieldlines screensaver
- fixed bug in this changelog (!)
- fixed bug in uninstall and verify scripts

* Mon Nov 18 2002 Michael Chapman <mchapman@student.usyd.edu.au> 0.2-1
- added cyclone screensaver

* Tue Nov 12 2002 Michael Chapman <mchapman@student.usyd.edu.au> 0.1-1
- initial version
